

#include <avr/io.h>
#include <util/delay.h>
//#include <stdlib.h>
#include <string.h>
#include <stdio.h>

void delay_ms(unsigned int ms)
/* delay for a minimum of <ms> */
{
  // we use a calibrated macro. This is more
  // accurate and not so much compiler dependent
  // as self made code.
  while(ms){
    _delay_ms(0.98);
    ms--;
  }
}


#if defined (__AVR_ATmega168__) || defined (__AVR_ATmega168A__)
// ATMEGA168
#define SPIPORT PORTB
#define SPIDDR  DDRB
#define SPIMISO 4       // Port B bit 5 (pin7): data in (data from MMC)
#define SPIMOSI 3       // Port B bit 6 (pin6): data out (data to MMC)
#define SPICLK  5       // Port B bit 7 (pin8): clock
#define SPICS   2       // Port B bit 4 (pin5: chip select for MMC
#endif

#if defined (__AVR_ATmega8U2__) || defined (__AVR_ATmega32U2__) || defined (__AVR_ATmega16U2__) ||defined (__AVR_AT90USB82__) || defined (__AVR_AT90USB162__) 
#define SPIPORT PORTB
#define SPIDDR  DDRB
#define SPIMISO 3       // PB3
#define SPIMOSI 2       // PB2
#define SPICLK  1       // PB1
#define SPICS   0       // PB0
#endif


unsigned char init; 

void SPIinit(void)
{
  SPIDDR &= ~_BV(SPIMISO);    // set port B SPI data input to input
  SPIDDR |=  _BV(SPICLK);    // set port B SPI clock to output
  SPIDDR |=  _BV(SPIMOSI);  // set port B SPI data out to output 
  SPIDDR |=  _BV(SPICS);   // set port B SPI chip select to output
  SPCR    =  _BV(SPE) | _BV(MSTR) | _BV(SPR1) | _BV(SPR0);
  SPIPORT |= _BV (SPICS); // set chip select to High (slave not active)
  init = 1; 
}

char SPI(char d) 
{  // send character over SPI
  char received = 0;
  SPDR = d;
  while(!(SPSR & _BV(SPIF)));
  received = SPDR;
  return (received);
}

void SPI_startpkt (void)
{
  SPIPORT &= ~_BV (SPICS); // set chip select to low(slave active)
}

void SPI_endpkt (void)
{
  SPIPORT |= _BV (SPICS); // set chip select to low(slave active)
}



static unsigned char addr = 0x82;

#define WAIT 10

void send_lcd (unsigned char rnum, unsigned char len, char *s)
{

  SPI_startpkt ();
  _delay_us (WAIT);
  SPI (addr);
  _delay_us (WAIT);
  SPI (rnum);
  _delay_us (WAIT);
  while (len--) {
    SPI (*s++);
    _delay_us (WAIT);
  }
  SPI_endpkt ();
}

void write_lcd (char *s)
{
  send_lcd (0, strlen (s), s);
}

void write_at_lcd (unsigned char x, unsigned char y, char *s)
{
  char c; 
  c = (y << 5) | (x & 0x1f);
  send_lcd (0x11, 1, &c);
  write_lcd (s);
}


int main (void)
{
  int h, m, s, i;
  char buf[10];

  DDRC=0x30;
  for (i=0;i<20;i++) {
    PINC = 0x30;
    delay_ms (250);    
  }
  

  SPIinit (); 
  h = 0;
  m = 0;
  s = 0;
  while (1) {
    sprintf (buf, "%02d:%02d:%02d", h, m, s);
    write_at_lcd (8, 1, buf);
    if (s < 59) s++;
    else {
      s = 0;
      if (m < 59) m++;
      else {
	m = 0;
	if (h < 23) h++;
	else {
	  h = 0;
	  // days ++;
	}
      }
    }
    delay_ms (1000);
  }
}
